function [onCount, onOFR, offCount, offOFR] = ELLAnalysis
% "singlePSTHs.m" is a function that plots the relevant plots of Fig 03
% and the corresponding figure supplement suppl. Fig 02 and Fig 06 of 
% the manuscript "Neural correlations enable invariant coding and 
% perception of natural stimuli" submitted to e-life (resubmission in 02/16)
%
% ___________________________
% Input
% during the excecution the location of the provided datafiles will need to be provided via an GUI
%
% ___________________________
% Output: 
% onCount         -       number of units (ON-units) that were included in the analysis
% onOFR            -       ongoing firing rates of included ON units 
% offCount         -       number of units (OFF-units) that were included in the analysis
% offOFR            -       ongoing firingrates of included OFF units
%
% ___________________________
% subfunctions:
% simmeas.m
% calcFI.m
%
% __________________________
% date : 2015-09-03
% last changes : 2016-02-26
% author : Volker Hofmann

%% %%%%%%%%%%%%%%%%%%%%%%
%% load filestructure & predifinitions
close all                                                                                                   % close all figure                                  
scrsz = get(0, 'screensize'); wid = scrsz(3)/4; hei = scrsz(4); offs = 50;       % obtain current screensize

dire = uigetdir('...\DATA-ELL\DATA');               % please select folder in which the provided mat files are located (e.g. "ELL-U001.mat")
dires = dir([dire, '\*.mat']);                             

OFRlimit = 100;          % all units with higher ongoing rates will be removed from the sample pool 
                                % ongoing firing rate is high for deep pyramidal cells and lower in intermediate 
                                % since feedback is only available to the latter two and seems to be neccessary 
                                % to decode correlations we restrict our analysis to these
                                
vorne = 0;                % defines beginning of evaluation window in s % -0.02;   
hinten= 0.03;            % defines end of evaluation window in s %6          


%% %%%%%%%%%%%%%%%%%%%%%%
%% read in data and build population average of ON-Cells

% initiate variables
onCount = 0;
onOFR = [];
onMHFR = [];
onBFR = [];
onPSTHS = cell(1,8);

% gather data
for i = 1:length(dires)
load([dire,'\', dires(i).name])
    if OnOff == 1 && OFR < OFRlimit   % analyze ON cells with a firing rate smaller than the defined limit
        onCount=onCount+1;
        onOFR=[onOFR, OFR];
        onMHFR=[onMHFR, MHFR];
        onBFR=[onBFR, BFR];
        
        % extract PSTHs in chirpwindow
        indZero = find(tPSTHs == min(abs(tPSTHs)));
        indStart = find(tPSTHs-vorne == min(abs(tPSTHs-vorne)));
        indEnd = find(tPSTHs-hinten == min(abs(tPSTHs-hinten)));
               
        tchirpPSTHs = tPSTHs(indStart : indEnd);
        PSTHs = PSTHs(:, indStart:indEnd);
              
        % stack on top of each other into a cell array
        for j = 1:size(PSTHs,1) 
            onPSTHS{j}=[onPSTHS{j}; PSTHs(j,:)];
        end
    end
end

% filter convolved single PSTHs
[B,A] = butter(1,0.1);
for i=1:size(onPSTHS,2) % go through different chirps
    for j=1:size(onPSTHS{1},1) % go through cells
        filtonPSTHS{i}(j,1:size(onPSTHS{1},2)) = filtfilt(B,A, onPSTHS{i}(j,:));
    end
end

% build population average for ON Cells
for i=1:size(onPSTHS,2)
     MeanOn(i,1:indEnd+1-indStart) = mean(onPSTHS{i});
     StdOn(i,1:indEnd+1-indStart) = std(onPSTHS{i});
     SemOn(i,1:indEnd+1-indStart) = (StdOn(i,:)./sqrt(size(onPSTHS{i},1)));
end

% filter On cell population averages for plotting 
[B,A] = butter(1,0.1);
for i = 1:size(MeanOn,1)
   filtMeanOn(i,1:size(MeanOn,2)) = filtfilt(B,A,MeanOn(i,:));
   filtSemOn(i,1:size(MeanOn,2)) = filtfilt(B,A,SemOn(i,:));
end


%% %%%%%%%%%%%%%%%%%%%%%%
%% visualization for ON cells 

% plot population averages for each chirp (Fig.3 - supplement 1; panel A)
for i=1:8
    h(i) = figure(i); figInd = i; hold on;%box on;
    if i<=4
            set(h(i), 'OuterPosition', [1+(figInd*wid)-wid hei-((hei-offs)/2)  wid (hei-offs)/2])  % arange psth figures
    elseif i>4
          figInd=  figInd-4;
          set(h(i), 'OuterPosition', [1+( figInd*wid)-wid hei-2*((hei-offs)/2)  wid (hei-offs)/2])  % arange psth figures
    end
    
    plot(tchirpPSTHs, filtMeanOn(i,:), 'b', 'LineWidth', 2)        
    plot([0 .03], [mean(onOFR), mean(onOFR)], '--b')
end

% pop waveforms ON into one plot (Fig 4; panel C)
figure(9); hold on
ind=1;
plus=[7 6 5 4]; minus = [3 2 1 8];

for i=[minus]
    hh(i)=  plot(tchirpPSTHs, filtMeanOn(i,:), 'k', 'LineWidth', 4);    
end

for i=[plus]
    hh(i)=  plot(tchirpPSTHs, filtMeanOn(i,:), 'k', 'LineWidth', 4);    
end

for i=[minus, plus]
    h(ind)=  plot(tchirpPSTHs, filtMeanOn(i,:), 'k', 'LineWidth', 4);    
    ind=ind+1;
end


set(h(5:8), 'Color', 'b'); set(h(1:4), 'Color', [.8 .8 1]); set(gca, 'Xlim', [vorne hinten], 'Ylim', [0 140])

%% %%%%%%%%%%%%%%%%%%%%%%
%% read in data and build population average of OFF-Cells

%initiate variables
offCount = 0;
offOFR = [];
offMHFR = [];
offBFR = [];
offPSTHS = cell(1,8);

% gather data
for i = 1:length(dires)
load([dire,'\', dires(i).name])
    if OnOff == 0 && OFR < OFRlimit     % analyze OFF cells with a firing rate smaller than the defined limit
        offCount=offCount+1;
        offOFR=[offOFR, OFR];
        offMHFR = [offMHFR, MHFR];
        offBFR = [offBFR, BFR];
    
        % extract PSTHs in chirpwindow
        indZero = find(tPSTHs == min(abs(tPSTHs)));
        indStart = find(tPSTHs-vorne == min(abs(tPSTHs-vorne)));
        indEnd = find(tPSTHs-hinten == min(abs(tPSTHs-hinten)));
       
        tchirpPSTHs = tPSTHs(indStart : indEnd);
        PSTHs = PSTHs(:, indStart:indEnd);
        
        % stack on top of each other in cellarray
        for j = 1:size(PSTHs,1) 
            offPSTHS{j}=[offPSTHS{j}; PSTHs(j,:)];
        end
    end
end

% filter convolved PSTHs
[B,A] = butter(1,0.1);
for i=1:size(offPSTHS,2) % go through different chirps
    for j=1:size(offPSTHS{1},1) % go through cells
        filtoffPSTHS{i}(j,1:size(offPSTHS{1},2)) = filtfilt(B,A, offPSTHS{i}(j,:));
    end
end

% build population averages for OFF Cells
for i=1:size(onPSTHS,2)
 MeanOff(i,1:indEnd+1-indStart) = mean(offPSTHS{i});
 StdOff(i,1:indEnd+1-indStart) = std(offPSTHS{i});
 SemOff(i,1:indEnd+1-indStart) = (StdOff(i,:)./sqrt(size(offPSTHS{i},1)));%*1.96;
end

% filter signals for plots
[B,A] = butter(1,0.1);
for i = 1:size(MeanOff,1)
   filtMeanOff(i,1:size(MeanOff,2)) = filtfilt(B,A,MeanOff(i,:));
   filtSemOff(i,1:size(MeanOff,2)) = filtfilt(B,A,SemOff(i,:));
end

%% %%%%%%%%%%%%%%%%%%%%%%
%% visualizations for OFF cells

% plot population averages for each chirp (Fig.3 - supplement 1; panel A)
for i=1:8
    h(i) = figure(i); figInd = i; hold on;% box on;
    if i<=4
            set(h(i), 'OuterPosition', [1+(figInd*wid)-wid hei-((hei-offs)/2)  wid (hei-offs)/2])  % arange psth figures
    elseif i>4
          figInd=  figInd-4;
          set(h(i), 'OuterPosition', [1+( figInd*wid)-wid hei-2*((hei-offs)/2)  wid (hei-offs)/2])  % arange psth figures
    end
    plot(tchirpPSTHs, filtMeanOff(i,:), 'r', 'LineWidth', 2)        
    plot([0 .03], [mean(offOFR), mean(offOFR) ], '--r')
end

% pop waveforms OFF into one plot (Fig.3; panel D)
figure(10); hold on
ind=1;
plus=[7 6 5 4];
minus = [3 2 1 8];

for i=[minus]
    hh(i)=  plot(tchirpPSTHs, filtMeanOff(i,:), 'k', 'LineWidth', 4);    
end

for i=[plus]
    hh(i)=  plot(tchirpPSTHs, filtMeanOff(i,:), 'k', 'LineWidth', 4);    
end

for i=[plus minus]
    h(ind) =  plot(tchirpPSTHs, filtMeanOff(i,:), 'k', 'LineWidth', 4);
    ind=ind+1;
end

set(h(5:8), 'Color', 'r'); set(h(1:4), 'Color', [1 .8 .8]); set(gca, 'Xlim', [vorne hinten], 'Ylim', [0 140])

%% scale figures 1-8
 
 xscale=.03;
 for i=1:8
 figure(i)
    plot( [0.029 0.029], [ 130, 130-mean(filtSemOn(i,:))], '-b', 'LineWidth', 2)
    plot( [0.028 0.028], [ 130, 130-mean(filtSemOff(i,:))], '-r', 'LineWidth', 2)
    set(gca, 'XLim', [0 xscale], 'YLim', [0 140])
 end

%% %%%%%%%%%%%%%%%%%%%%%%
%% cross correlation of popavgs (Fig.3 - supplement 1; panel B)
 
 for i = 1:size(filtMeanOff,1)
 [r, p]=corrcoef(filtMeanOff(i,:),filtMeanOn(i,:));
 rs(i)=r(1,2);
 ps(i)=p(1,2);
 end
 figure
 bar(rs)
 set(gca, 'YLim', [-1 .2])
 
%% %%%%%%%%%%%%%%%%%%%%%%
%% tuning curve

win =.015/2;
winpos = .015;
plus=[7 6 5 4];
minus = [3 2 1 8];
for i=1:size(filtoffPSTHS,2) % go through different chirps
    for j=1:size(filtoffPSTHS{1},1) % go through cells
        if mean (filtoffPSTHS{i}(j, tchirpPSTHs>winpos-win & tchirpPSTHs<winpos+win)') >.05
            newOffTuning(j,i) = log(mean (filtoffPSTHS{i}(j, tchirpPSTHs>winpos-win & tchirpPSTHs<winpos+win)')./offBFR(j));
        else
            newOffTuning(j,i) = NaN;
        end
    end
end
 
onOFR(onOFR==0)=1;
for i=1:size(filtonPSTHS,2) % go through different chirps
    for j=1:size(filtonPSTHS{1},1) % go through cells
        if mean(filtonPSTHS{i}(j, tchirpPSTHs>winpos-win & tchirpPSTHs<winpos+win)') >.05
        newOnTuning(j,i) = log(mean (filtonPSTHS{i}(j, tchirpPSTHs>winpos-win & tchirpPSTHs<winpos+win)')./onBFR(j));
        else
            newOnTuning(j,i) = NaN;
        end
    end
end

figure  % tuning curves (Fig. 3; panel E)
 hold on 
 errorbar(1:8, nanmean(newOnTuning(:,[plus, minus])), nanstd(newOnTuning(:,[plus, minus]))/sqrt(length(onOFR)) , 'g')
 errorbar(1:8, nanmean(newOffTuning(:,[plus, minus])), nanstd(newOffTuning(:,[plus, minus]))/sqrt(length(offOFR)), 'r')
    set(gca, 'YLim', [-3.2 2.5])
 
figure % normalized tuning curves (Fig. 6; panel B)
hold on
    plot(1:8, nanmean(newOnTuning(:,[plus, minus]))./max(abs(nanmean(newOnTuning(:,[plus, minus])))) , 'g')
    plot(1:8, nanmean(newOffTuning(:,[plus, minus]))./max(abs(nanmean(newOffTuning(:,[plus, minus])))) , 'r')
    set(gca, 'YLim', [-1 1])

%% %%%%%%%%%%%%%%%%%%%%%%
%% calculate phase invariance invariance
 
  %compute FIs for chirps [8,1,2,3] and [4:7] separately
  % get chirp waveforms for chirp-window
  indZero = find(wave_Chirps_t == min(abs(wave_Chirps_t)));
  indEnd =  find(wave_Chirps_t-0.03 == min(abs(wave_Chirps_t-0.03)));
  wave_Chirps_win = wave_Chirps(: ,indZero:indEnd);
  wave_Chirps_t_win = wave_Chirps_t(: ,indZero:indEnd);
 
  plus=[4:7];
  minus=[1:3,8];
  
% calc FI for ONs 
[FI_all_ON, SimRespoON, SimChirpsON] = calcFI(filtonPSTHS, wave_Chirps_win, plus, minus); title('ON cells')

figure; hold on;
plot(mean(SimChirpsON), SimRespoON(7,:), 'ob', 'MarkerFaceColor', 'b')  % Example from ON cell #7
set(gca, 'YLim', [.1 1], 'Yscale', 'log', 'XLim', [.1, 1.25], 'Xscale' , 'log')
refline(1,0)
xlabel('Distance stimulus'); ylabel('Distance response'); title('ON cells')


% calc FI for OFFs
[FI_all_OFF, SimRespoOFF, SimChirpsOFF] =  calcFI(filtoffPSTHS, wave_Chirps_win, plus, minus); title('OFF cells')

figure; hold on;
plot(mean(SimChirpsOFF), SimRespoOFF(6,:), 'or', 'MarkerFaceColor', 'r')  % Example from OFF cell #6
set(gca, 'YLim', [.1 1], 'Yscale', 'log', 'XLim', [.1, 1.25], 'Xscale' , 'log')
refline(1,0)
xlabel('Distance stimulus'); ylabel('Distance response'); title('OFF cells')

%  p = anova1([FI_all_ON';FI_all_OFF'], [zeros(length(FI_all_ON),1);zeros(length(FI_all_OFF),1)+1])
