function [FI_all, simRespoAll, simChirpsAll] = calcFI(psths, chirps, plus, minus)
% "calcFI.m" is a subfunction of "ELLAnalysis.m". Please refer to help of main function
% for additional information 
% submitted to e-life (resubmission in 02/16)
%
% ___________________________
% Input:
% psths             -       cell array holding PSTHs of cells to analyze each cell should contain data recorded to one type of chirp, while within each cell each row is the data of one unit
% chirps            -       matrix with chrip waveforms during the evaluated window. The different waveforms are stacked as rows. 
% plus               -       vector specifying the cells in "psths" that are categorized as "plus chirps"
% minus            -       vector specifying the cells in "psths" that are categorized as "minus chirps"
%
% ___________________________
% Output: 
% FI_all             -       FI score calculated across all chirps 
% simChirpAll    -       matrix holding the the distance of input waveforms. Each row holds data of one comparison
% simRespoAll   -       matrix holding the the distance of responses. Each row holds data of on cell
% __________________________
% date : 2016-02-20
% last changes : 2016-02-26
% author : Volker Hofmann


%% initialize variables 
 FI_intra_P  = NaN (1, size(psths{1},1)); 
 FI_intra_M = NaN (1, size(psths{1},1)); 
 FI_inter = NaN (1, size(psths{1},1)); 
 FI_all = NaN (1, size(psths{1},1)); 
 
 simRespoAll = NaN (size(psths{1},1), size(nchoosek([minus,plus],2),1));
 simChirpsAll = NaN (size(psths{1},1), size(nchoosek([minus,plus],2),1));

 
%% go through data of each unit separately
for i = 1:size(psths{1},1)
    
    %% get PSTHs of current cell
    currPSTHs= NaN(size(psths,2),size(psths{1},2));
    for j = 1:size(psths,2)  % number of chirps
        currPSTHs(j,:) = psths{j}(i,:);
    end
    
    %% calc FI score chirps within one category (Intra Minus / --)
    
    comb = nchoosek(minus,2);             % get combinations among - chirps

    simChirps = NaN (1, size(comb,1)); simRespo = simChirps;
    for ii = 1:size(comb,1)                     % calculate similarity of combinations
        [~,simChirps(ii)]=simmeas(chirps(comb(ii,1),:), chirps(comb(ii,2),:));
        [~,simRespo(ii)]=simmeas(currPSTHs(comb(ii,1),:), currPSTHs(comb(ii,2),:));
    end
    
    ratio = simRespo./simChirps;
    ratio(ratio>1)=1;
    
    FI_intra_M(i) = 1-(sum(ratio)/length(ratio));
    clear comb simRespo simChirps ratio ii
    
    %% calc FI score chirps within one category (Intra Plus / ++)
    
    comb = nchoosek(plus,2);             % get combinations among - chirps
    
    simChirps = NaN (1, size(comb,1)); simRespo = simChirps;
    for ii = 1:size(comb,1)                     % calculate similarity of combinations
        [~,simChirps(ii)]=simmeas(chirps(comb(ii,1),:), chirps(comb(ii,2),:));
        [~,simRespo(ii)]=simmeas(currPSTHs(comb(ii,1),:), currPSTHs(comb(ii,2),:));
    end
    
    ratio = simRespo./simChirps;
    ratio(ratio>1)=1;
    
    FI_intra_P(i) = 1-(sum(ratio)/length(ratio));
    clear comb simRespo simChirps ratio ii
    
    %% calc FI score between chirps of different categories (Inter / +-)
    
    comb = combvec(minus,plus)';      % get combination of each minus chirp to each plus chirp
    
    simChirps = NaN (1, size(comb,1)); simRespo = simChirps;
    for ii = 1:size(comb,1)   % calculate similarity between all possible chirp & response combinations
        [~,simChirps(ii)]=simmeas(chirps(comb(ii,1),:), chirps(comb(ii,2),:));
        [~,simRespo(ii)]=simmeas(currPSTHs(comb(ii,1),:), currPSTHs(comb(ii,2),:));
    end
    
    ratio = simRespo./simChirps;
    ratio(ratio>1)=1;
    
    FI_inter(i) = 1-(sum(ratio)/length(ratio));
    clear comb simRespo simChirps ratio
    
    %% calc FI score between all chirps (no sorting / overall data) 
   
    comb = nchoosek([minus,plus],2);  % get all possible combination between plus and minus chirps
    
    simChirps = NaN (1, size(comb,1)); simRespo = simChirps;
    for ii = 1:size(comb,1)     % calculate similarity between all possible chirp & response combinations
        [~,simChirps(ii)]=simmeas(chirps(comb(ii,1),:), chirps(comb(ii,2),:));
        [~,simRespo(ii)]=simmeas(currPSTHs(comb(ii,1),:), currPSTHs(comb(ii,2),:));
        
        simRespoAll(i,ii)=simRespo(ii);  % gather data for output
        simChirpsAll(i,ii)=simChirps(ii);
    end
    
    ratio = simRespo./simChirps;
    ratio(ratio>1)=1;
    
    FI_all(i) = 1-(sum(ratio)/length(ratio));
    clear comb ratio
end


%% visualization 
figure;
bar([mean(FI_all), mean([FI_intra_P, FI_intra_M]), mean(FI_inter)])
labels = {'all', '++ & --', '+-'};
set(gca, 'YLim', [0 1], 'XTickLabels' , labels)
hold on;
errorbar([mean(FI_all), mean([FI_intra_P, FI_intra_M]), mean(FI_inter)], [std(FI_all)./sqrt(length(FI_all)), std([FI_intra_P, FI_intra_M])./sqrt(length([FI_intra_P, FI_intra_M])), std(FI_inter)./sqrt(length(FI_inter))],  'LineStyle', 'none')

%optional one way anova comparison of categories
%p = anova1([[FI_intra_P, FI_intra_M]';FI_inter'], [zeros(length([FI_intra_P, FI_intra_M]),1);zeros(length(FI_inter),1)+1])

end